@@ -1,15 +1,24 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
-/**********************************************************************
-
-  rbgobject.h -
-
-  $Author: sakai $
-  $Date: 2007/07/19 22:03:44 $
-
-  Copyright (C) 2003,2006  Ruby-GNOME2 Project Team
-  Copyright (C) 2002,2003  Masahiro Sakai
-
-**********************************************************************/
+/*
+ *  Copyright (C) 2011  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2003,2006  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002,2003  Masahiro Sakai
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA  02110-1301  USA
+ */
 
 #ifndef __RBGOBJECT_H__
 #define __RBGOBJECT_H__
@@ -36,8 +45,10 @@ extern "C" {
     G_DEF_CLASS_WITH_GC_FUNC(gtype, name, module, mark, free)
 #define G_DEF_CLASS3(gtype_name, name, module)\
     (rbgobj_define_class_dynamic(gtype_name, name, module, 0, 0))
-#define G_DEF_CLASS4(gtype, name, module, parent) \
+#define G_DEF_CLASS_WITH_PARENT(gtype, name, module, parent) \
     (rbgobj_define_class(gtype, name, module, 0, 0, parent))
+#define G_DEF_CLASS4(gtype, name, module, parent) \
+    G_DEF_CLASS_WITH_PARENT(gtype, name, module, parent)
 
 #define G_DEF_INTERFACE(gtype, name, module)\
     (rbgobj_define_class(gtype, name, module, 0, 0, Qnil))
@@ -87,6 +98,7 @@ RUBY_GLIB2_VAR  ID rbgobj_id_children;
 #define GOBJ2RVAL_UNREF(gobj) (rbgobj_ruby_object_from_instance_with_unref(gobj))
 #define GOBJ2RVALU(gobj) GOBJ2RVAL_UNREF(gobj)
 #define GVAL2RVAL(v)    (rbgobj_gvalue_to_rvalue(v))
+#define GVAL2RVAL_UNSET(v) (rbgobj_gvalue_to_rvalue(v))
 
 #define RVAL2BOXED(obj, gtype)  (rbgobj_boxed_get(obj, gtype))
 #define BOXED2RVAL(cobj, gtype) (rbgobj_make_boxed(cobj, gtype))
@@ -175,7 +187,7 @@ extern GType rbgobj_gtype_get(VALUE obj);
 
 /* rbgobj_signal.c */
 typedef VALUE (*GValToRValSignalFunc)(guint num,const GValue* values);
-extern void rbgobj_set_signal_func(VALUE klass, gchar* sig_name, GValToRValSignalFunc func);
+extern void rbgobj_set_signal_func(VALUE klass, const gchar *sig_name, GValToRValSignalFunc func);
 extern GValToRValSignalFunc rbgobj_get_signal_func(guint signal_id);
 extern VALUE rbgobj_signal_wrap(guint sig_id);
 
@@ -188,6 +200,7 @@ extern void g_rclosure_set_tag(GClosure *closure, const gchar *tag);
 
 /* rbgobj_value.c */
 extern VALUE rbgobj_gvalue_to_rvalue(const GValue* value);
+extern VALUE rbgobj_gvalue_to_rvalue_unset(GValue *value);
 extern void rbgobj_rvalue_to_gvalue(VALUE val, GValue* result);
 extern void rbgobj_initialize_gvalue(GValue *result, VALUE value);
 
@@ -203,7 +216,7 @@ extern VALUE rbgobj_ptr_new(GType type, gpointer ptr);
 extern gpointer rbgobj_ptr2cptr(VALUE ptr);
 
 #define RBGOBJ_TYPE_RUBY_VALUE (rbgobj_ruby_value_get_type())
-extern GType rbgobj_ruby_value_get_type();
+extern GType rbgobj_ruby_value_get_type(void);
 extern VALUE g_value_get_ruby_value(const GValue* value);
 extern void g_value_set_ruby_value(GValue* value, VALUE ruby);
 
@@ -214,7 +227,11 @@ extern void rbgobj_register_property_getter(GType gtype, const char* prop_name,
 /* rbgobj_boxed.c */
 extern VALUE rbgobj_boxed_create(VALUE klass); /* deprecated */
 extern gpointer rbgobj_boxed_get(VALUE obj, GType gtype);
+extern gpointer rbgobj_boxed_get_default(VALUE obj, GType gtype);
 extern VALUE rbgobj_make_boxed(gpointer data, GType gtype);
+extern VALUE rbgobj_make_boxed_raw(gpointer p, GType gtype,
+                                   VALUE klass, gint flags);
+extern VALUE rbgobj_make_boxed_default(gpointer data, GType gtype);
 extern void rbgobj_boxed_not_copy_obj(GType gtype);
 
 /* rbgobj_enums.c */
@@ -254,17 +271,19 @@ extern GType g_poll_fd_get_type(void);
 typedef struct {
     GType type;
     VALUE klass;
-    VALUE (*get_superclass)(void);
-    void (*type_init_hook)(VALUE);
-    void (*rvalue2gvalue)(VALUE val, GValue *result);
-    VALUE (*gvalue2rvalue)(const GValue *);
-    void (*initialize)(VALUE, gpointer);
-    gpointer (*robj2instance)(VALUE);
-    VALUE (*instance2robj)(gpointer);
-    void (*unref)(gpointer instance);
+    gpointer user_data;
+    GDestroyNotify notify;
+    VALUE (*get_superclass)(gpointer user_data);
+    void (*type_init_hook)(VALUE klass, gpointer user_data);
+    void (*rvalue2gvalue)(VALUE value, GValue *result, gpointer user_data);
+    VALUE (*gvalue2rvalue)(const GValue *value, gpointer user_data);
+    void (*initialize)(VALUE rb_instance, gpointer insntance, gpointer user_data);
+    gpointer (*robj2instance)(VALUE rb_instance, gpointer user_data);
+    VALUE (*instance2robj)(gpointer instance, gpointer user_data);
+    void (*unref)(gpointer instance, gpointer user_data);
 } RGConvertTable;
 
-extern void rbgobj_convert_define(RGConvertTable *table);
+extern void rbgobj_convert_define(const RGConvertTable *table);
 
 #ifdef __cplusplus
 }
