@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2011  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2010-2013  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -210,6 +210,14 @@ module Milter
           milter_conf.packet_buffer_size = size
         end
 
+        @option_parser.on("--max-pending-finished-sessions=N",
+                          Integer,
+                          "Don't hold over N pending finished sessions.",
+                          "(#{milter_conf.max_pending_finished_sessions})") do |n|
+          raise OptionParser::InvalidArgument if n <= 0
+          milter_conf.max_pending_finished_sessions = n
+        end
+
         @option_parser.on("--[no-]handle-signal",
                           "Handle SIGHUP, SIGINT and SIGTERM signals",
                           "(#{milter_conf.handle_signal?})") do |boolean|
