@@ -2,10 +2,11 @@
 #
 # Author:: Nathaniel Talbott.
 # Copyright:: Copyright (c) 2000-2002 Nathaniel Talbott. All rights reserved.
-#             Copyright (c) 2009-2010 Kouhei Sutou. All rights reserved.
+#             Copyright (c) 2009-2013 Kouhei Sutou. All rights reserved.
 # License:: Ruby license.
 
 require 'test/unit'
+require "testunit-test-util"
 
 module Test
   module Unit
@@ -1640,6 +1641,12 @@ EOM
         end
       end
 
+      def test_pass_minus_expected
+        check_nothing_fails do
+          assert_in_epsilon(-1, -1)
+        end
+      end
+
       def test_fail_with_message
         check_fail("message.\n" +
                     "<10000> -/+ (<10000> * <0.1>)[1000.0] " +
