@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2008-2013  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -245,6 +245,14 @@ class TestConfigurationLoader < Test::Unit::TestCase
     assert_equal(65535, @configuration.chunk_size)
   end
 
+  def test_manager_max_pending_finished_sessions
+    assert_equal(0, @configuration.max_pending_finished_sessions)
+    @loader.manager.max_pending_finished_sessions = 29
+    assert_equal(29, @configuration.max_pending_finished_sessions)
+    @loader.manager.max_pending_finished_sessions = nil
+    assert_equal(0, @configuration.max_pending_finished_sessions)
+  end
+
   def test_database_type
     assert_equal(nil, @configuration.database.type)
     @loader.database.type = "mysql"
