@@ -97,6 +97,7 @@ static gchar *actual_helo_fqdn;
 static gchar *actual_envelope_from;
 static GList *actual_recipients;
 static GString *actual_body_string;
+static MilterHeaders *expected_headers;
 static MilterHeaders *actual_headers;
 
 typedef void (*EndOfMessageActionFunc) (void);
@@ -341,7 +342,7 @@ cb_header (MilterCommandDecoder *decoder, const gchar *name, const gchar *value,
         send_reply(MILTER_COMMAND_HEADER, test_data);
 
     n_headers++;
-    milter_headers_add_header(actual_headers, name, value);
+    milter_headers_append_header(actual_headers, name, value);
 }
 
 static void
@@ -471,6 +472,7 @@ cut_setup (void)
     actual_helo_fqdn = NULL;
     actual_envelope_from = NULL;
     actual_recipients = NULL;
+    expected_headers = milter_headers_new();
     actual_headers = milter_headers_new();
     actual_body_string = g_string_new(NULL);
     actual_defined_macros =
@@ -514,6 +516,8 @@ cut_teardown (void)
         g_list_foreach(actual_recipients, (GFunc)g_free, NULL);
         g_list_free(actual_recipients);
     }
+    if (expected_headers)
+        g_object_unref(expected_headers);
     if (actual_headers)
         g_object_unref(actual_headers);
     if (actual_body_string)
@@ -1488,48 +1492,120 @@ option_test_assert_body (void)
 }
 
 static void
-option_test_assert_mail_file (void)
+option_test_assert_mail_file_lf (void)
 {
-    MilterHeader expected_header1 = {"Return-Path", "<xxxx@example.com>"};
-    MilterHeader expected_header2 = {"Date", "Wed, 5 Nov 2008 11:41:54 -0200"};
-    MilterHeader expected_header3 = {"Message-ID", "<168f979z.5435357@xxx>"};
-    MilterHeader expected_header4 = {"Content-Type", "text/plain; charset=\"us-ascii\""};
-    MilterHeader expected_header5 = {"X-Source", ""};
-    MilterHeader expected_header6 = {"To", "Test Sender <test-to@example.com>"};
-    MilterHeader expected_header7 = {"From", "test-from@example.com"};
-    MilterHeader expected_header8 = {"List-Archive",
-                                     "\n <http://sourceforge.net/mailarchive/forum.php?forum_name=milter-manager-commit>"};
-    MilterHeader expected_header9 = {"Subject", "This is a test mail"};
-    MilterHeader expected_header10 = {"List-Help",
-                                      "\n <mailto:milter-manager-commit-request@lists.sourceforge.net?subject=help>"};
-
     cut_assert_equal_string("<test-from@example.com>", actual_envelope_from);
     gcut_assert_equal_list_string(
         gcut_take_new_list_string("<test-to@example.com>",
                                   NULL),
         actual_recipients);
 
-    cut_assert_equal_uint(10, milter_headers_length(actual_headers));
-    milter_assert_equal_header(&expected_header1,
-                               milter_headers_get_nth_header(actual_headers, 1));
-    milter_assert_equal_header(&expected_header2,
-                               milter_headers_get_nth_header(actual_headers, 2));
-    milter_assert_equal_header(&expected_header3,
-                               milter_headers_get_nth_header(actual_headers, 3));
-    milter_assert_equal_header(&expected_header4,
-                               milter_headers_get_nth_header(actual_headers, 4));
-    milter_assert_equal_header(&expected_header5,
-                               milter_headers_get_nth_header(actual_headers, 5));
-    milter_assert_equal_header(&expected_header6,
-                               milter_headers_get_nth_header(actual_headers, 6));
-    milter_assert_equal_header(&expected_header7,
-                               milter_headers_get_nth_header(actual_headers, 7));
-    milter_assert_equal_header(&expected_header8,
-                               milter_headers_get_nth_header(actual_headers, 8));
-    milter_assert_equal_header(&expected_header9,
-                               milter_headers_get_nth_header(actual_headers, 9));
-    milter_assert_equal_header(&expected_header10,
-                               milter_headers_get_nth_header(actual_headers, 10));
+    milter_headers_append_header(expected_headers,
+                                 "Return-Path",
+                                 "<xxxx@example.com>");
+    milter_headers_append_header(expected_headers,
+                                 "Date",
+                                 "Wed, 5 Nov 2008 11:41:54 -0200");
+    milter_headers_append_header(expected_headers,
+                                 "Message-ID",
+                                 "<168f979z.5435357@xxx>");
+    milter_headers_append_header(expected_headers,
+                                 "Content-Type",
+                                 "text/plain; charset=\"us-ascii\"");
+    milter_headers_append_header(expected_headers,
+                                 "X-Source",
+                                 "");
+    milter_headers_append_header(expected_headers,
+                                 "To",
+                                 "Test Sender <test-to@example.com>");
+    milter_headers_append_header(expected_headers,
+                                 "From",
+                                 "test-from@example.com");
+    milter_headers_append_header(expected_headers,
+                                 "List-Archive",
+                                 "\n <http://sourceforge.net/mailarchive/forum.php?forum_name=milter-manager-commit>");
+    milter_headers_append_header(expected_headers,
+                                 "Subject",
+                                 "This is a test mail");
+    milter_headers_append_header(expected_headers,
+                                 "List-Help",
+                                 "\n <mailto:milter-manager-commit-request@lists.sourceforge.net?subject=help>");
+    milter_headers_append_header(expected_headers,
+                                 "Authentication-Results",
+                                 "mail.example.com;\n"
+                                 "\tspf=none smtp.mailfrom=test-from@example.com;\n"
+                                 "\tsender-id=none header.From=test-from@example.com;\n"
+                                 "\tdkim=none;\n"
+                                 "\tdkim-adsp=none header.From=test-from@example.com");
+    milter_headers_append_header(expected_headers,
+                                 "Authentication-Results",
+                                 "mail.example.org;\n"
+                                 "\tspf=none smtp.mailfrom=test-from@example.com;\n"
+                                 "\tsender-id=none header.From=test-from@example.com;\n"
+                                 "\tdkim=none;\n"
+                                 "\tdkim-adsp=none header.From=test-from@example.com");
+
+    milter_assert_equal_headers(expected_headers, actual_headers);
+
+    cut_assert_equal_string("Message body\nMessage body\nMessage body\n",
+                            actual_body_string->str);
+}
+
+static void
+option_test_assert_mail_file_crlf (void)
+{
+    cut_assert_equal_string("<test-from@example.com>", actual_envelope_from);
+    gcut_assert_equal_list_string(
+        gcut_take_new_list_string("<test-to@example.com>",
+                                  NULL),
+        actual_recipients);
+
+    milter_headers_append_header(expected_headers,
+                                 "Return-Path",
+                                 "<xxxx@example.com>");
+    milter_headers_append_header(expected_headers,
+                                 "Date",
+                                 "Wed, 5 Nov 2008 11:41:54 -0200");
+    milter_headers_append_header(expected_headers,
+                                 "Message-ID",
+                                 "<168f979z.5435357@xxx>");
+    milter_headers_append_header(expected_headers,
+                                 "Content-Type",
+                                 "text/plain; charset=\"us-ascii\"");
+    milter_headers_append_header(expected_headers,
+                                 "X-Source",
+                                 "");
+    milter_headers_append_header(expected_headers,
+                                 "To",
+                                 "Test Sender <test-to@example.com>");
+    milter_headers_append_header(expected_headers,
+                                 "From",
+                                 "test-from@example.com");
+    milter_headers_append_header(expected_headers,
+                                 "List-Archive",
+                                 "\r\n <http://sourceforge.net/mailarchive/forum.php?forum_name=milter-manager-commit>");
+    milter_headers_append_header(expected_headers,
+                                 "Subject",
+                                 "This is a test mail");
+    milter_headers_append_header(expected_headers,
+                                 "List-Help",
+                                 "\r\n <mailto:milter-manager-commit-request@lists.sourceforge.net?subject=help>");
+    milter_headers_append_header(expected_headers,
+                                 "Authentication-Results",
+                                 "mail.example.com;\r\n"
+                                 "\tspf=none smtp.mailfrom=test-from@example.com;\r\n"
+                                 "\tsender-id=none header.From=test-from@example.com;\r\n"
+                                 "\tdkim=none;\r\n"
+                                 "\tdkim-adsp=none header.From=test-from@example.com");
+    milter_headers_append_header(expected_headers,
+                                 "Authentication-Results",
+                                 "mail.example.org;\r\n"
+                                 "\tspf=none smtp.mailfrom=test-from@example.com;\r\n"
+                                 "\tsender-id=none header.From=test-from@example.com;\r\n"
+                                 "\tdkim=none;\r\n"
+                                 "\tdkim-adsp=none header.From=test-from@example.com");
+
+    milter_assert_equal_headers(expected_headers, actual_headers);
 
     cut_assert_equal_string("Message body\r\nMessage body\r\nMessage body\r\n",
                             actual_body_string->str);
@@ -1561,11 +1637,34 @@ option_test_assert_output_message (void)
         normalized_output_string->str);
 }
 
+static void
+option_test_assert_output_message_charset (void)
+{
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  Content-Type: text/plain; charset=\"us-ascii\"" "\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
+}
+
 static void
 option_test_assert_large_mail (void)
 {
-    const gchar expected_header[] = "\r\n--=-fXjrl3xPRPdvOSpDlnzU\r\n";
-    const gchar expected_footer[] = "\r\n--=-fXjrl3xPRPdvOSpDlnzU--\r\n";
+    const gchar expected_header[] = "\n--=-fXjrl3xPRPdvOSpDlnzU\n";
+    const gchar expected_footer[] = "\n--=-fXjrl3xPRPdvOSpDlnzU--\n";
 
     cut_assert_equal_memory(expected_header, strlen(expected_header),
                             actual_body_string->str, strlen(expected_header));
@@ -1574,7 +1673,7 @@ option_test_assert_large_mail (void)
                             actual_body_string->len -
                             strlen(expected_footer),
                             strlen(expected_footer));
-    cut_assert_equal_int(2435493, actual_body_string->len);
+    cut_assert_equal_int(2404259, actual_body_string->len);
 }
 
 static void
@@ -1670,9 +1769,16 @@ data_option (void)
     ADD("output-message",
         "--output-message",
         option_test_assert_output_message);
-    ADD("mail-file",
-        mail_file_option("parse-test.mail"),
-        option_test_assert_mail_file);
+    ADD("output-message - charset",
+        "--header='Content-Type:text/plain; charset=\"us-ascii\"' "
+        "--output-message",
+        option_test_assert_output_message_charset);
+    ADD("mail-file - LF",
+        mail_file_option("parse-test-lf.mail"),
+        option_test_assert_mail_file_lf);
+    ADD("mail-file - CRLF",
+        mail_file_option("parse-test-crlf.mail"),
+        option_test_assert_mail_file_crlf);
     ADD("large-file",
         mail_file_option("large.mail"),
         option_test_assert_large_mail);
